	 /************************************************/
	 /*                                              */
	 /*       HOSTCOMM.C - Host Communication        */
	 /*                    with T44P controller      */
	 /*                                              */
	 /*            Copyright (c) 1991                */
	 /*            Triangle Research                 */
	 /************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

void init_serial();
int serial_getc(void);
int transfer(void);
int retrieve(void);
int sendblock(unsigned char *s,int blocksize);
int receiveblock(unsigned char *s,int blocksize);
void comm_error(void);
void serial_out(unsigned char ch);
int serial_in();
int serial_status();

int commport,commstatus,commdata, commcontrol;

void main()
{
  char command[20], response[100];
  char ch;
  int i;

  printf("\nCOM port (1 or 2)? >");
  scanf("%d",&ch);
  if (ch==1)  commport = 0;    /* commport = 0 for COM1: */
  else commport = 1;           /* commport = 1 for COM2: */

  ch=getchar(); 	       /* clear keyboard buffer */
  init_serial();               /* initialize PC's serial port */
  clrscr();                    /* clear PC's screen */
  printf("       Send Point-to-point command to T44P\n");
  printf("      ====================================\n");

  while (1) {
    printf("Command String ? ");
    gets(command);
    if (send_command(command, response)==-1) comm_error();
    else printf("Response String: %s\n\n",response);
  }
}

/*-------------------------------------------------------------------------*/
/* This function establish a Point-to-point communication session with the */
/* target controller by sending Ctrl-E character and obtain echo of Ctrl-E */
/* It will subsequently send the command "string" via the serial port to   */
/* the controller. It will then wait to receive the response block and     */
/* store in the "response" string. If there is error communicating with    */
/* the controller it will return (-1).                                     */
/*-------------------------------------------------------------------------*/

int send_command(char *string,char *response)
{
    int i;
    int ch;
    serial_out(5);
    if ((ch=serial_getc()) == -1)
	return (-1);
    for (i=0;string[i]!=0;i++) serial_out(string[i]);
    serial_out('\x0D');

    for (i=0;i<100;i++)
    {
       if ((ch=serial_getc()) == -1)
	  return (-1);
       response[i]=ch;
       if (ch=='*') {
	  ch=serial_getc();
	  response[i+1]=0;
	  break;
       }
    }
    return (1);
}
/*-------------------------------------------------------------------------*/
/* This function check the serial port "commport" (COM1 = 0; COM2 = 1) for */
/* data. It returns the 7-bit data if ready.                               */
/* If after 100000 times still no data ready, function return (-1) to       */
/* indicate time-out error. This is to prevent locking up the computer in  */
/* a dead loop if device does not respond.                                 */
/*-------------------------------------------------------------------------*/

int serial_getc(void)
{
   int i,j;
   for (j=0;j<10;j++) {
     for (i=0;i<10000;i++) {                 /* allow for time out */
	if (serial_status())
	    return (serial_in());
     }
   }
   return(-1);                           /* nothing received, time-out !! */
}

/*-------------------------------------------------------------------------*/
/* Output a character to the serial port                                   */
/*-------------------------------------------------------------------------*/

void serial_out(unsigned char ch) {
  while (!(inportb(commstatus) & 0x20))
       ;
  outportb(commdata,ch);
}

/*-------------------------------------------------------------------------*/
/*  Initialize the IBMPC serial communication port to 9600, 8 data bits,   */
/*  1 stop bit and no parity.                                              */
/*  must be called once before communicating with T44P                     */
/*-------------------------------------------------------------------------*/

#define SETTINGS (0x03|0x00|0x00|0xE0)

void init_serial(void)
{
  bioscom(0,SETTINGS,commport);
  if (commport==0)
  {
     commstatus = 0x3FD;
     commdata   = 0x3F8;
     commcontrol = 0x3FC;
  }
  else
  {
     commstatus = 0x2FD;
     commdata   = 0x2F8;
     commcontrol = 0x2FC;
  }
  outportb(commcontrol,1);    /* Force /DTR to active for use by Opto232 */
  while (serial_in());        /* clear any characters in serial buffer */
}
/*-------------------------------------------------------------------------*/
/*  If a character is ready at the 8250 UART, return the character.        */
/*  Otherwise return 0.                                                    */
/*-------------------------------------------------------------------------*/

int serial_in() {
  if (inportb(commstatus) & 0x01) return(inportb(commdata));
  return(0);
}
/*-------------------------------------------------------------------------*/
/*  Check 8250 UART status.                                                */
/*    returns 1 if a character is present, otherwise returns 0             */
/*-------------------------------------------------------------------------*/

int serial_status() {
  return (inportb(commstatus) & 0x01);
}
/*-------------------------------------------------------------------------*/
/*  Report to user that a communication error occur between the computer   */
/*  and the controller                                                     */
/*-------------------------------------------------------------------------*/

void comm_error(void)
{
  int k;
  printf(
    "\n\tCommunication Error!... Check serial port COM%d:\n\n",commport+1);
}
